<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class NudgifySocialProofPixelDataModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        $action = Tools::getValue('action', 'all');
        $productId = Tools::getValue('product');

        $pixelData = [];

        if (in_array($action, ['all', 'cart'])) {
            $this->loadCart($pixelData);
        }

        if (in_array($action, ['all', 'product']) && $productId) {
            $this->loadProduct($productId, $pixelData);
        }

        ob_end_clean();
        header('Content-Type: application/json');
        die(Tools::jsonEncode(['data' => $pixelData]));
    }

    private function loadCart(&$pixelData)
    {
        $cart = $this->context->cart;
        $currency = new CurrencyCore($cart->id_currency);

        $pixelData['cart'] = [
            'amount' => $cart->getOrderTotal(),
            'currency' => $currency->iso_code,
        ];
    }

    private function loadProduct($productId, &$pixelData)
    {
        $product = new Product($productId);

        if (!$product) {
            return;
        }

        $pixelData['product'] = [
            'id' => $product->id,
            'stock' => $product->quantity,
            'image' => $this->getProductImageUrl($product),
        ];

        $pixelData['combinations'] = array_map(function ($combo) {
            return [
                'id_product_attribute' => $combo['id_product_attribute'],
                'id_attribute' => $combo['id_attribute'],
                'stock' => $combo['quantity']
            ];
        }, $product->getAttributeCombinations($this->context->language->id));
    }

    private function getProductImageUrl($product)
    {
        $coverImage = $product->getCover($product->id);

        return $this->context->link->getImageLink(
            isset($product->link_rewrite) ? $product->link_rewrite : $product->name,
            $coverImage['id_image'],
            ImageType::getFormatedName('small')
        );
    }
}
