<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class NudgifySocialProof extends Module
{
    protected $config_form = false;
    protected $nudgify_settings = [];

    public function __construct()
    {
        $this->name = 'nudgifysocialproof';
        $this->tab = 'advertising_marketing';
        $this->version = '1.1.0';
        $this->author = 'Convertize';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Nudgify - Social Proof');
        $this->description = $this->l('Nudgify is a complete Social Proof App & FOMO App for e-Commerce');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall Nudgify?');
        $this->module_key = 'd8c1314a54f4fb3a2bbc9853d48289d0';

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);

        
        #STATIC_SETTINGS_START:DONOT_REMOVE_THIS_COMMENT
        $this->nudgify_settings = json_decode('{
    "SENTRY_DSN": "",
    "INTEGRATION_NAME": "prestashop",
    "PIXEL_URL": "https:\/\/pixel.nudgify.com\/pixel.js",
    "AUTH_ENDPOINT": "https:\/\/app.nudgify.com\/api\/connect",
    "WEBHOOK_ENDPOINT": "https:\/\/collect.nudgify.com\/api\/prestashop\/webhook",
    "SYNC_ENDPOINT": "https:\/\/collect.nudgify.com\/api\/prestashop\/sync"
}', true);
        #STATIC_SETTINGS_END:DONOT_REMOVE_THIS_COMMENT
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('NUDGIFY_ENABLED', true);
        Configuration::updateValue('NUDGIFY_SITE_KEY', '');
        Configuration::updateValue('NUDGIFY_API_KEY', '');
        Configuration::updateValue('NUDGIFY_AUTO_SYNC_ORDERS_ENABLED', true);

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('actionValidateOrder');
    }

    public function uninstall()
    {
        Configuration::deleteByName(
            'NUDGIFY_ENABLED',
            'NUDGIFY_SITE_KEY',
            'NUDGIFY_API_KEY',
            'NUDGIFY_AUTO_SYNC_ORDERS_ENABLED'
        );

        return parent::uninstall();
    }

    /** Load the configuration form */
    public function getContent()
    {
        if ((bool)Tools::isSubmit('nudgify_configuration_submitted')) {
            if (Tools::getValue('nudgify_setup_connection')) {
                $this->postProcess();
                $this->setupConnection();
            }

            if (Tools::getValue('nudgify_save')) {
                $this->postProcess();
                $this->showAdminInfo($this->l('Settings saved'));
            }

            if (Tools::getValue('nudgify_manually_sync')) {
                $this->syncOrdersManually();
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $this->renderForm() . $output;
    }

    /** Create the form that will be displayed in the configuration of your module. */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'nudgify_configuration_submitted';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm($this->getConfigForm());
    }

    /** Create the structure of your form. */
    protected function getConfigForm()
    {
        $forms = array();

        $forms[] = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Connection Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable Nudgify'),
                        'name' => 'NUDGIFY_ENABLED',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Yes')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('No')
                            )
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'desc' => $this->l('Site Key'),
                        'name' => 'NUDGIFY_SITE_KEY',
                        'class' => 'depends-on-enabled',
                        'label' => $this->l('Site Key'),
                        'desc' => '
                            Go to the <a href="https://app.nudgify.com/integrations/prestashop" target="_blank" 
                            rel="noopener">Prestashop Integration</a> page to find your Site Key.
                        ',
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'desc' => '
                            Go to your <a href="https://app.nudgify.com/settings" target="_blank" 
                            rel="noopener">Profile Details</a> to find your API Key.
                        ',
                        'name' => 'NUDGIFY_API_KEY',
                        'class' => 'depends-on-enabled',
                        'label' => $this->l('API Key'),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Sync new orders to Nudgify'),
                        'name' => 'NUDGIFY_AUTO_SYNC_ORDERS_ENABLED',
                        'is_bool' => true,
                        'class' => 'depends-on-enabled',
                        'desc' => $this->l('Enable this option to automatically sync new orders with Nudgify.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Set up connection'),
                    'name' => 'nudgify_setup_connection',
                    'id' => 'nudgify_setup_connection',
                    'class' => 'btn btn-default nudgify-btn',
                ),
                'buttons' => array(
                    '0' => array(
                        'type' => 'submit',
                        'value' => 1,
                        'title' => $this->l('Manually Sync Order'),
                        'name' => 'nudgify_manually_sync',
                        'id' => 'nudgify_do_manually_sync',
                        'class' => 'hidden nudgify-btn',
                    ),
                    '1' => array(
                        'type' => 'submit',
                        'value' => 1,
                        'title' => $this->l('Save settings'),
                        'name' => 'nudgify_save',
                        'id' => 'nudgify_save',
                        'class' => 'nudgify-btn',
                    ),
                )
            ),
        );

        return $forms;
    }

    protected function getConfigFormValues()
    {
        return array(
            'NUDGIFY_ENABLED' => Configuration::get('NUDGIFY_ENABLED', false),
            'NUDGIFY_SITE_KEY' => Configuration::get('NUDGIFY_SITE_KEY', ''),
            'NUDGIFY_API_KEY' => Configuration::get('NUDGIFY_API_KEY', ''),
            'NUDGIFY_AUTO_SYNC_ORDERS_ENABLED' => Configuration::get('NUDGIFY_AUTO_SYNC_ORDERS_ENABLED', true),
        );
    }

    /** Save form data. */
    protected function postProcess()
    {
        $formValues = $this->getConfigFormValues();

        foreach (array_keys($formValues) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCss($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        if (!Configuration::get('NUDGIFY_ENABLED', false)) {
            return;
        }

        $siteKey = Configuration::get('NUDGIFY_SITE_KEY', '');
        if (!$siteKey) {
            return;
        }

        $cart = $this->context->cart;

        $product = null;
        if (get_class($this->context->controller) === 'ProductController') {
            $product = $this->context->controller->getProduct();
        }

        $pixelData = ['data' => []];

        if ($cart) {
            $currency = new CurrencyCore($cart->id_currency);
            $pixelData['data']['cart'] = [
                'amount' => $cart->getOrderTotal(),
                'currency' => $currency->iso_code,
            ];
        }

        if ($product) {
            $pixelData['data']['product'] = [
                'id' => $product->id,
                'stock' => $product->quantity,
                'image' => $this->getProductImageUrl($product),
            ];

            $combinationImages = $this->getProductCombinationImages($product);

            $pixelData['data']['combinations'] = array_map(function ($combo) use ($combinationImages) {
                $hasNoCombo = empty($combinationImages[$combo['id_product_attribute']]);
                return [
                    'id_product_attribute' => $combo['id_product_attribute'],
                    'id_attribute' => $combo['id_attribute'],
                    'stock' => $combo['quantity'],
                    'image' => $hasNoCombo ? null : $combinationImages[$combo['id_product_attribute']]
                ];
            }, $product->getAttributeCombinations($this->context->language->id));
        }

        $this->context->smarty->assign('pixelData', json_encode($pixelData));
        $this->context->smarty->assign('siteKey', $siteKey);
        $this->context->smarty->assign('pixelUrl', $this->getSetting('PIXEL_URL'));
        $this->context->smarty->assign('dataUrl', $this->getDataUrlBase());

        return $this->context->smarty->fetch($this->local_path . '/views/templates/frontend/header.tpl');
    }

    public function hookActionValidateOrder($params)
    {
        if (!Configuration::get('NUDGIFY_ENABLED', false)) {
            return;
        }

        if (!Configuration::get('NUDGIFY_AUTO_SYNC_ORDERS_ENABLED', true)) {
            return;
        }

        $orderData = $this->buildOrderData($params['order']);
        $orderData['ip'] = Tools::getRemoteAddr();

        $this->post($this->getSetting('WEBHOOK_ENDPOINT'), $orderData);
    }

    public function setupConnection()
    {
        $siteKey = Configuration::get('NUDGIFY_SITE_KEY', '');
        $apiKey = Configuration::get('NUDGIFY_API_KEY', '');

        if (!$this->validCredentials($siteKey, $apiKey)) {
            return;
        }

        $response = $this->post($this->getSetting('AUTH_ENDPOINT'), array(
            "site_key" => Configuration::get('NUDGIFY_SITE_KEY', ''),
            "integration_identity" => $this->getSetting('INTEGRATION_NAME')
        ));

        if ($response['status']) {
            $this->showAdminInfo($this->l('Connection is working properly'));
        } else {
            $this->showAdminError($this->l($response['message']));
        }
    }

    public function syncOrdersManually()
    {
        if (!Configuration::get('NUDGIFY_ENABLED', false)) {
            $this->showAdminError($this->l('Please Enable Nudgify to Sync your orders'));
            return;
        }

        $siteKey = Configuration::get('NUDGIFY_SITE_KEY');
        $apiKey = Configuration::get('NUDGIFY_API_KEY');

        if (!$this->validCredentials($siteKey, $apiKey)) {
            return;
        }

        $orders = Order::getOrdersWithInformations(30);

        $data = [
            'site_key' => $siteKey,
            'orders' => []
        ];

        foreach ($orders as $order) {
            $data['orders'][] = $this->buildOrderData(new Order($order['id_order']));
        }

        $response = $this->post($this->getSetting('SYNC_ENDPOINT'), $data);

        if ($response['status']) {
            $this->showAdminInfo($this->l('Orders synced successfully'));
        } else {
            $this->showAdminError($this->l($response['message']));
        }
    }

    protected function getDataUrlBase()
    {
        return $this->context->link->getModuleLink($this->name, 'pixeldata', ['ajax' => true]);
    }

    protected function getLineItems($order)
    {
        $products = [];

        foreach ($order->getProducts() as $productItem) {
            $product = new Product($productItem['product_id']);
            $products[] = [
                'item_id' => $product->id,
                'item_variation_id' => null,
                'item_name' => $this->getProductName($product),
                'item_link' => $this->getProductUrl($product),
                'image_url' => $this->getProductImageUrl($product),
            ];
        }

        return $products;
    }

    protected function getProductName($product)
    {
        if (is_array($product->name)) {
            return reset($product->name);
        }

        return $product->name;
    }

    protected function getProductUrl($product)
    {
        return $this->context->link->getProductLink($product);
    }

    protected function getProductImageUrl($product)
    {
        $coverImage = $product->getCover($product->id);

        return $this->context->link->getImageLink(
            isset($product->link_rewrite) ? $product->link_rewrite : $product->name,
            $coverImage['id_image'],
            ImageType::getFormatedName('small')
        );
    }

    protected function getProductCombinationImages($product)
    {
        $combinationImages = $product->getCombinationImages($this->context->language->id);
        $output = [];

        if (empty($combinationImages)) {
            return [];
         }
                 
        foreach ($combinationImages as $idProductAttribute => $combinationImage) {
            $output[$idProductAttribute] = $this->context->link->getImageLink(
                isset($product->link_rewrite) ? $product->link_rewrite : $product->name,
                $combinationImage[0]['id_image'],
                ImageType::getFormatedName('small')
            );
        }

        return $output;
    }

    protected function buildOrderData($order)
    {
        $customer = $order->getCustomer();
        $address = new Address($order->id_address_invoice);
        $state = new State($address->id_state);
        $country = new Country($address->id_country);

        $customerName = implode(' ', [
            $customer->firstname,
            $customer->lastname
        ]);

        return [
            'site_key' => Configuration::get('NUDGIFY_SITE_KEY'),
            'order_id' => $order->id,
            'email' => $customer->email,
            'name' => $customerName,
            'city' => $address->city,
            'state' =>  $state->name ?? null,
            'country' => $country->iso_code,
            'ip' => null,
            'date' => $this->getOrderDate($order),
            'line_items' => $this->getLineItems($order)
        ];
    }

    protected function showAdminError($message)
    {
        $this->context->controller->errors[] = $message;
    }

    protected function showAdminInfo($message)
    {
        $this->context->controller->confirmations[] = $message;
    }

    protected function post($url, $data)
    {
        $header = array();
        $header = [
            "Content-Type:application/json",
            "Authorization: Bearer " . Configuration::get('NUDGIFY_API_KEY'),
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        $response = curl_exec($ch);
        $errors = curl_error($ch);
        curl_close($ch);

        return [
            'message' => trim($response),
            'errors' => $errors,
            'status' => empty($errors) ? true : false
        ];
    }

    protected function validCredentials($siteKey, $apiKey)
    {
        $UUIDv4 = '/^[0-9A-F]{8}-[0-9A-F]{4}-4[0-9A-F]{3}-[89AB][0-9A-F]{3}-[0-9A-F]{12}$/Di';

        $validSiteKey = preg_match($UUIDv4, $siteKey);
        $validApiKey = Tools::strlen($apiKey) === 60;

        if (!$validSiteKey) {
            $this->showAdminError($this->l('Please supply a valid Site Key'));
        }

        if (!$validApiKey) {
            $this->showAdminError($this->l('Please supply a valid API Key'));
        }

        return ($validSiteKey && $validApiKey);
    }

    protected function getOrderDate($order)
    {
        $timestamp = strtotime($order->date_add);

        return gmdate('Y-m-d H:i:s', $timestamp);
    }

    protected function getSetting($key)
    {
        return $this->nudgify_settings[$key] ?? null;
    }
}
